package jp.co.sds_corp.fxchart.graph;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.RenderingHints;
import java.awt.Stroke;
import java.awt.image.BufferedImage;
import java.io.OutputStream;
import java.text.DecimalFormat;

import jp.co.sds_corp.fxutil.CcyPair;
import jp.co.sds_corp.nanchart.graph.XScale;
import jp.co.sds_corp.nanchart.graph.YScale;
import jp.co.sds_corp.util.common.CmnDate;

/**
* 為替チャートクラスの基底クラスです。
*/
public abstract class RateGraph
{
	String	ccy1;
	String	ccy2;
	String	term = "SP";
	String	span;
	String	ma;
	CmnDate	from;
	CmnDate	to;
	int	width = -1;
	int	height = -1;
	IRateSource	source;
	CcyPair	pair;
	Image	bgimg;
	Quote	quote;
	Stroke	lineStroke;
	Color	lineColor;
	String	copyright;
	int	marginHeight = 25;
	int	marginWidth = 25;

	/**
	* 通貨1(ccy1)を設定します。
	* @param	var		設定するデータ
	*/
	public void setCcy1( String var ) {
		this.ccy1 = var ;
	}
	/**
	* 通貨1(ccy1)を取得します。
	* @return	ccy1の内容
	*/
	public String getCcy1() {
		return ccy1;
	}

	/**
	* 通貨2(ccy2)を設定します。
	* @param	var		設定するデータ
	*/
	public void setCcy2( String var ) {
		this.ccy2 = var ;
	}
	/**
	* 通貨2(ccy2)を取得します。
	* @return	ccy2の内容
	*/
	public String getCcy2() {
		return ccy2;
	}

	/**
	* 種別(span)を設定します。
	* @param	var		設定するデータ
	*/
	public void setSpan( String var ) {
		this.span = var ;
	}
	/**
	* 種別(span)を取得します。
	* @return	spanの内容
	*/
	public String getSpan() {
		return span;
	}

	/**
	* 移動平均種別(ma)を設定します。
	* @param	var		設定するデータ
	*/
	public void setMa( String var ) {
		this.ma = var ;
	}
	/**
	* 移動平均種別(ma)を取得します。
	* @return	maの内容
	*/
	public String getMa() {
		return ma;
	}

	/**
	* 期間開始日(from)を設定します。
	* @param	var		設定するデータ
	*/
	public void setFrom( CmnDate var ) {
		this.from = var ;
	}
	/**
	* 期間開始日(from)を取得します。
	* @return	fromの内容
	*/
	public CmnDate getFrom() {
		return from;
	}

	/**
	* 期間終了日(to)を設定します。
	* @param	var		設定するデータ
	*/
	public void setTo( CmnDate var ) {
		this.to = var ;
	}
	/**
	* 期間終了日(to)を取得します。
	* @return	toの内容
	*/
	public CmnDate getTo() {
		return to;
	}

	/**
	* 幅(width)を設定します。
	* @param	var		設定するデータ
	*/
	public void setWidth( int var ) {
		this.width = var ;
	}
	/**
	* 幅(width)を取得します。
	* @return	widthの内容
	*/
	public int getWidth() {
		return width;
	}

	/**
	* 高さ(height)を設定します。
	* @param	var		設定するデータ
	*/
	public void setHeight( int var ) {
		this.height = var ;
	}
	/**
	* 高さ(height)を取得します。
	* @return	heightの内容
	*/
	public int getHeight() {
		return height;
	}

	/**
	* レートソース(source)を設定します。
	* @param	var		設定するデータ
	*/
	public void setSource( IRateSource var ) {
		this.source = var ;
	}
	/**
	* レートソース(source)を取得します。
	* @return	sourceの内容
	*/
	public IRateSource getSource() {
		return source;
	}

	/**
	* 公表相場(quote)を設定します。
	* @param	var		設定するデータ
	*/
	public void setQuote( Quote var ) {
		this.quote = var ;
	}
	/**
	* 公表相場(quote)を取得します。
	* @return	quoteの内容
	*/
	public Quote getQuote() {
		return quote;
	}

	/**
	* 通貨ペア(pair)を設定します。
	* @param	var		設定するデータ
	*/
	public void setPair( CcyPair var ) {
		this.pair = var ;
	}
	/**
	* 通貨ペア(pair)を取得します。
	* @return	pairの内容
	*/
	public CcyPair getPair() {
		return pair;
	}

	/**
	* 背景の画像(bgimg)を設定します。
	* @param	var		設定するデータ
	*/
	public void setBgImage( Image var ) {
		this.bgimg = var;
	}
	/**
	* 背景の画像(bgimg)を取得します。
	* @return	pairの内容
	*/
	public Image getBgImage() {
		return bgimg;
	}

	/**
	* 線の種類(lineStroke)を設定します。
	* @param	var		設定するデータ
	*/
	public void setLineStroke( Stroke var ) {
		this.lineStroke = var ;
	}
	/**
	* 線の種類(lineStroke)を取得します。
	* @return	lineStrokeの内容
	*/
	public Stroke getLineStroke() {
		if ( lineStroke == null ) {
			lineStroke = new BasicStroke( 1 );
		}
		return lineStroke;
	}

	/**
	* 線の色(lineColor)を設定します。
	* @param	var		設定するデータ
	*/
	public void setLineColor( Color var ) {
		this.lineColor = var ;
	}
	/**
	* 線の色(lineColor)を取得します。
	* @return	lineColorの内容
	*/
	public Color getLineColor() {
		if ( lineColor == null ) {
			lineColor = Color.gray;
		}
		return lineColor;
	}

	/**
	* 著作権表示(copyright)を設定します。
	* @param	var		設定するデータ
	*/
	public void setCopyright( String var ) {
		this.copyright = var ;
	}
	/**
	* 著作権表示(copyright)を取得します。
	* @return	copyrightの内容
	*/
	public String getCopyright() {
		return copyright;
	}

	/**
	* 高さのマージン(marginHeight)を設定します。
	* @param	var		設定するデータ
	*/
	public void setMarginHeight( int var ) {
		this.marginHeight = var ;
	}
	/**
	* 高さのマージン(marginHeight)を取得します。
	* @return	marginHeightの内容
	*/
	public int getMarginHeight() {
		return marginHeight;
	}

	/**
	* 幅のマージン(marginWidth)を設定します。
	* @param	var		設定するデータ
	*/
	public void setMarginWidth( int var ) {
		this.marginWidth = var ;
	}
	/**
	* 幅のマージン(marginWidth)を取得します。
	* @return	marginWidthの内容
	*/
	public int getMarginWidth() {
		return marginWidth;
	}


	public void start( OutputStream out ) throws Exception {
		if ( width == 0 || height == 0 ) {
			width = 450;
			height = 310;
		}
		makeGraph( out );
	}

	protected abstract void makeGraph( OutputStream out ) throws Exception;

	protected void setXScale( XScale xs, String span, CmnDate fr, CmnDate to )
	{
		Font font = new Font( "Dialog", Font.PLAIN, 8 );
		CmnDate tmp = (CmnDate )fr.clone();
		tmp.set( CmnDate.SECOND, 0 );
		for ( ;; ) {
			tmp.add( CmnDate.MINUTE, 1 );
			if ( tmp.after( to ) ) {
				break;
			}
			if (( tmp.get( CmnDate.MINUTE ) % 10 ) == 0 )
			{
				xs.add( tmp.clone(), tmp.format( "HH:mm" ));
			}
		}
		xs.setLineStroke( new BasicStroke( 1 ));
		xs.setLineColor( Color.gray );
		xs.setScaleLine( true );
		xs.setFont( font );
	}

	protected void setYScale( YScale ys,
					double dMin, double dMax, double dDiv, CcyPair pair )
	{
		if ( dMax < 0.0 ) {
			return;
		}
		String pattern = "##0.0000000000".substring( 0, pair.getPoint()+4 );
		DecimalFormat df = new DecimalFormat( pattern );

		Font font = new Font( "Dialog", Font.PLAIN, 8 );
		ys.setFont( font );
		String sc;
		sc = df.format( dMin );
		ys.add( dMin, sc );
		sc = df.format( dMax );
		ys.add( dMax, sc );

		int unit = 1;
		if (( dMax - dMin ) > ( 15 / dDiv )) {
			unit = 5;
			if (( dMax - dMin ) > ( 75 / dDiv )) {
				unit = 10;
			}
			if (( dMax - dMin ) > ( 150 / dDiv )) {
				unit = 50;
			}
			if (( dMax - dMin ) > ( 750 / dDiv )) {
				unit = 100;
			}
			if (( dMax - dMin ) > ( 1500 / dDiv )) {
				unit = 500;
			}
		}
		for ( int i = (int )(dMin*dDiv)+1; i <= (int )(dMax*dDiv); i++ ) {
			if (( i % unit ) != 0 ) {
				continue;
			}
			sc = df.format( (double)i / dDiv );
			if ( sc.endsWith( "." )) {
				sc = sc.substring( 0, sc.length() - 1 );
			}
			ys.add( (double)i / dDiv, sc );
		}
		ys.setLineStroke( getLineStroke() );
		ys.setLineColor( getLineColor() );
		ys.setScaleLine( true );
	}
	
	protected void copyright( BufferedImage img, int x, int y ) {
		if ( getCopyright() == null ) {
			return;
		}
		Graphics2D gr = img.createGraphics();
		gr.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
								RenderingHints.VALUE_ANTIALIAS_ON);
		gr.setColor( Color.black );
		gr.setFont( new Font( "DialogInput", Font.PLAIN, 10 ));
		gr.drawString( getCopyright(), x, y );
	}

}
